# -*- Import: -*-
from paver.easy import *
from paver.setuputils import setup
#from setuptools import setup
from setuptools import find_packages
import sys, os
sys.path.append('./src')
import neuronvisio

try:
    # Optional tasks, only needed for development
    # -*- Optional import: -*-
    from github.tools.task import *
    import paver.doctools
    import paver.virtual
    import paver.misctasks
    ALL_TASKS_LOADED = True
except ImportError, e:
    info("some tasks could not not be imported.")
    debug(str(e))
    ALL_TASKS_LOADED = False



version = neuronvisio.__version__

classifiers = [
    # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
    "Development Status :: 4 - Beta",
    "Programming Language :: Python",
    "Topic :: Scientific/Engineering :: Visualization"

    ]

install_requires = [
    # -*- Install requires: -*-
    'setuptools',
    ]

entry_points="""
    # -*- Entry points: -*-
    """

# compatible with distutils of python 2.3+ or later
setup(
    name='neuronvisio',
    version=version,
    description='NeuronVisio is a Graphical User Interface for NEURON simulator enviroment',
    long_description=open('README.rst', 'r').read(),
    packages = ['neuronvisio', 'neuronvisio.modeldb'],
    package_dir={'neuronvisio': 'src/neuronvisio'},
    package_data=paver.setuputils.find_package_data(package="neuronvisio", ),
    scripts= ['src/nrnvisio.py'],
    classifiers=classifiers,
    keywords='neuron, gui, pylab, 3D, visualization',
    author='Michele Mattioni',
    author_email='mattioni@ebi.ac.uk',
    url='http://mattions.github.com/neuronvisio/',
    license='GPLv3',
    include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    entry_points=entry_points,
    )

options(
    # -*- Paver options: -*-
    minilib=Bunch(
        extra_files=[
            # -*- Minilib extra files: -*-
            ]
        ),
    sphinx=Bunch(
        docroot='docs',
        builddir="_build",
        sourcedir=""
        ),
    virtualenv=Bunch(
        packages_to_install=[
            # -*- Virtualenv packages to install: -*-
            'github-tools',
            "nose",
            "Sphinx>=0.6b1",
            "pkginfo", 
            "virtualenv"],
        dest_dir='./virtual-env/',
        install_paver=True,
        script_name='bootstrap.py',
        paver_command_line=None
        ),
    )

#options.setup.package_data=paver.setuputils.find_package_data(
#    'src', package='neuronvisio', only_in_packages=False)

if ALL_TASKS_LOADED:
    @task
    @needs('generate_setup', 'minilib', 'gh_pages_build_fix', 
           'setuptools.command.sdist')
    def sdist():
        """Overrides sdist to make sure that our setup.py is generated."""
        print "Package baked."
        
@task
@needs(['gh_pages_build'])
def gh_pages_build_fix():
    """Generate the html and fix the _images with image
    
    Bug submitted: http://github.com/dinoboff/github-tools/issues#issue/10
    """
    import glob
    import os.path
    
    root_dir = 'docs/_build/html/'
    for filename in glob.glob(os.path.join(root_dir, '*.html')):
        f = open(filename, 'r')
        buffer = ""
        try:
            for line in f:
                buffer += line.replace('_images', 'images')
        except:
            pass
        f.close()
        # reopen and rewriting
        f = open(filename, 'w')
        f.write(buffer)
        f.close()
    print "html documentation fixed."
    
@task
def build_pdf():
    """Build the User Manual"""
    from subprocess import call
    root = os.getcwd()
    docs = os.path.join(root, 'docs')
    pdf_building = os.path.join(docs, '_build/pdf/')
    manual_filename = 'Neuronvisio_User_Manual.pdf'
    print "root dir: %s, docs dir: %s" %(root, docs)
    
    # building latex
    call(['sphinx-build', '-b', 'latex', docs, pdf_building])
    
    # building pdf
    os.chdir(pdf_building)
    call(['pdflatex', 'neuronvisio.tex'])
    # second round for the toc
    call(['pdflatex', 'neuronvisio.tex'])
    
    #copying the file
    os.rename(os.path.join(pdf_building, 'neuronvisio.pdf'), 
              os.path.join(docs, manual_filename))
    print "Usual Manual created in the docs dir"

@task
def build_source_deb():
    """Build a source debian package of the version supplied"""
    from subprocess import call
    import os.path
    package_ver = neuronvisio.__version__
    dist_dir = 'dist'
    package_name = 'neuronvisio' + '-' + package_ver + '.tar.gz'
    tarball = os.path.join('dist', package_name)
    call(['py2dsc', tarball])
    os.chdir(dist_dir)
    call(['debuild', '-S', '-sa'])
    os.chdir('..')

#@task
#def upload_to_launchpad():
#    package_ver = neuronvisio.__version__
#    dist_dir = 'dist'
#    package_src_changes = 'neuronvisio_' + package_ver + '_source.changes'
#    call(['dput', 'neuronvisio-ppa', os.path.join('deb_dist', 
#                                                  package_src_changes)])



# -*- coding: utf-8 -*-

import os

from pkg_resources import parse_version
import pkginfo

def _egg_info(path_to_egg='../'):
    path_to_egg = os.path.join(
        os.path.dirname(__file__), path_to_egg)
    egg_info = pkginfo.Develop(path_to_egg)
    release = egg_info.version
    parsed_version = parse_version(release)
    version = '%s.%s' % tuple([int(x) for x in parsed_version[0:2]])
    return egg_info.name, egg_info.author, version, release

project, author, version, release = _egg_info()
copyright = '2009, Michele Mattioni'

# Extension
extensions = ['sphinx.ext.autodoc', 'sphinx.ext.intersphinx', 'sphinx.ext.todo', 
'sphinx.ext.coverage', 'sphinx.ext.pngmath', 'sphinx.ext.ifconfig', 
'sphinx.ext.autosummary', 'github.tools.sphinx']

intersphinx_mapping = {'http://docs.python.org/': None}
htmlhelp_basename = 'Neuronvisiodoc'
# Source
master_doc = 'index'
templates_path = ['_templates']
source_suffix = '.rst'
exclude_trees = []
pygments_style = 'sphinx'


# -- Options for HTML output ---------------------------------------------------

# The theme to use for HTML and HTML Help pages.  Major themes that come with
# Sphinx are currently 'default' and 'sphinxdoc'.
html_theme = 'sphinxdoc'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#html_theme_options = {}

# Add any paths that contain custom themes here, relative to this directory.
html_theme_path = ['_template']

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
#html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
#html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
#html_logo = None

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
#html_favicon = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static', 'images']

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
#html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
#html_additional_pages = {}

# If false, no module index is generated.
#html_use_modindex = True

# If false, no index is generated.
#html_use_index = True

# If true, the index is split into individual pages for each letter.
#html_split_index = False

# If true, links to the reST sources are added to the pages.
#html_show_sourcelink = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# If nonempty, this is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = ''

# Output file base name for HTML help builder.
htmlhelp_basename = '%sdoc' % project

# -- Options for LaTeX output --------------------------------------------------

# The paper size ('letter' or 'a4').
latex_paper_size = 'a4'

# The font size ('10pt', '11pt' or '12pt').
#latex_font_size = '10pt'

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass [howto/manual]).

# latex build settings
latex_documents = [
    ('index', '%s.tex' % project, u'%s Documentation' % project,
    author, 'manual'),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
#latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
#latex_use_parts = False

# Additional stuff for the LaTeX preamble.
#latex_preamble = ''

# Documents to append as an appendix to all manuals.
#latex_appendices = []

# If false, no module index is generated.
#latex_use_modindex = True

# Extension Options
autoclass_content='both'



# This example comes with NeuronVisio.

# In this example we show how to load a model written in hoc with neruonvisio and explore
# it.

# The model that I choose is: 
# A model of spike initiation in neocortical pyramidal neurons 
# Z. F. Mainen, J. Joerges, J. R. Huguenard and T. J. Sejnowski 
# Neuron 15, 1427-1439 (1995)

# From the original demo I deleted the synapses and the interview panel for clarity.


# You need to compile the hoc file before launching it
# $ nrnivmodl 

# Importing the NeuronVisio
from neuronvisio.controls import Controls
controls = Controls()

# Importing hoc interpreter
from neuron import h

# loading the model
# importing the interview so the GUI does not freeze
# Uncomment this if you want to use also the interview GUI
#import neuron.gui

# Load the script
h.load_file("demo.hoc")


## Insert an IClamp
st = h.IClamp(0.5, sec=h.soma)
st.amp = 0.25
st.delay = 3
st.dur = 40



# * Copyright (C) Thu Jul  9 11:08:09 BST 2009 - Michele Mattioni:
# *  
# * This file is part of NeuronVisio
# * 
# * NeuronVisio is free software: you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation, either version 3 of the License, or
# * (at your option) any later version.
#
# * NeuronVisio is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
#
# * You should have received a copy of the GNU General Public License
# * along with NeuronVisio.  If not, see <http://www.gnu.org/licenses/>.

""":synopsis: Manage the map between vectors and sections"""

from neuron import h
import numpy as np
import os
import tables
import datetime
import matplotlib
import matplotlib.pyplot as plt
import logging
logger = logging.getLogger(__name__)


class Manager(object):
    """The Manager class can be used as a library and imported on cluster 
    simulation. Refer to the online doc for how to save in HDF5 and how to 
    create New Generic Ref.
    """

    def __init__(self):
        
        self.groups = {}
#        self.vecRefs = [] 
#        self.synVecRefs = []
        self.refs = {}
        self.results_root = 'results'
        self.geometry_root = 'geometry'
        self.geometry_node_name = 'geom'        
        # Load the std run for NEURON
        h.load_file("stdrun.hoc")
        
    def add_vecRef(self, var, sec, 
                   time_interval_recording=None, 
                   point_process=None):
        """Add the vecRef to the vec_res list. It takes care to create the vector 
        and record the given variable.
        
        :param var: The variable to record
        :param sec: The section where to record
        :param time_interval_recording: If None, use the dt of Neuron. 
                    Specify which interval to use to record the vectors. 
                    Note the same resolution should apply to the time 
                    vectors.
        :return: True if the vector is created successfully.
        
        """
            
        success = False
        if hasattr(sec, var):
            # Adding the vector only if does not exist
            alreadyPresent=False
            if self.refs.has_key('VecRef'):
                for vecRef in self.refs['VecRef']:
                    if vecRef.sec_name == sec.name():
                        if vecRef.vecs.has_key(var):
                            alreadyPresent = True
                            break
                        else: # Adding a variable to an existing vecRef
                            vec = h.Vector()
                            varRef = '_ref_' + var
                            if time_interval_recording is None:
                                vec.record(point_process, 
                                           getattr(sec(0.5), varRef))
                            else:
                                vec.record(point_process,
                                           getattr(sec(0.5), varRef), 
                                           time_interval_recording)
                            vecRef.vecs[var] = vec
                            alreadyPresent = True
                            success = True
                 
            if not alreadyPresent:
                    
                # Creating the vector
                vec = self.create_record_vector(sec, 
                                                var, 
                                                time_interval_recording)
                                                
                # Adding to the list
                vecRef = VecRef(sec)
                vecRef.vecs[var] = vec
                name = vecRef.__class__.__name__
                if self.groups.has_key('t'):
                    t = self.groups['t']
                else: 
                    t = self.create_time_record(time_interval_recording)
                self.add_ref(vecRef, t)
                success = True
        return success
    
    def add_all_vecRef(self, var, 
                       time_interval_recording=None, 
                       point_process=None):
        """Create the vector for all the section present in the model 
        with the given variable
        :param var: The variable to record"""
        
        done = False
        responses = []
        for sec in h.allsec():
            response = self.add_vecRef(var, 
                                       sec, 
                                       time_interval_recording,
                                       point_process)
            responses.append(response)
        if all(responses) != True:
            logger.warning( "Warning: Some vectors could not be added.")
    
    
    def add_ref(self, generic_ref, x):
        """Add a generic ref to manager.refs dictionary. If a list of the ref of 
        the same type is already present, the `genercref` will be added, otherwise the list
        will be created.
        `x` is the indipendent variable which should be used to when the genericref is 
        plotted from the Neuronvisio UI. 
        
        `generic_ref` -- the ref to add to the manager.ref list
        `x` -- indipendent varialbe use to plot the variable from the genericref"""
        name = generic_ref.group_id
        if self.refs.has_key(name):
            self.refs[name].append(generic_ref)
        else:
            self.refs[name] = [generic_ref]
            self.groups[name] = x
            
    def add_synVecRef(self, synapse):
        """Add the synVecRef object to the list
        
        :param synapse: The synapse to record.
        """
        synVecRef = SynVecRef(synapse.chan_type, synapse.section.name(), 
                              synapse.vecs)
            
        self.add_ref(synVecRef, self.groups['t'])
    
    
    
    def find_point_process(self, sec):
        """Find a point_process in a section if any.
        
        Params:
            sec - Section to search
            
        Return:
            point_process or None if not present.
        """
        mt = h.MechanismType(1)
        total_mech = int(mt.count())
        sec.push()
        pp = None
        for i in range(total_mech):
            pp_type = mt.select(i)
            pp = mt.pp_begin()
            if pp is not None:
                break
            
        h.pop_section()
        return pp

    def create_record_vector(self, sec, var, time_interval_recording):
        """Create the vector which will record the variable
        Will pass the first pointprocess to make sure the recording 
        are thread safe."""
        
        # Getting the first pp if any
        
        point_process = self.find_point_process(sec)
            
        vec = h.Vector()
        varRef = '_ref_' + var
        
        
        try:
            if time_interval_recording:
                if point_process:
                    vec.record(point_process, 
                               getattr(sec(0.5), varRef), 
                               time_interval_recording)
                    logging.info( "Sec: %s has pp: %s" %(sec.name(), point_process))
                else:
                    vec.record(getattr(sec(0.5), varRef), 
                               time_interval_recording)
                    logging.info( "Sec: %s has not pp" %(sec.name()))
            else:
                if point_process:
                    vec.record(point_process, 
                               getattr(sec(0.5), varRef))
                else:
                    vec.record(getattr(sec(0.5), varRef))
                
            
        except NameError:
            logging.error( "The variable %s is not present in the section %s" \
            % (varRef, sec.name()))
        
        h.pop_section()
        return vec
    
    def create_time_record(self, time_interval_recording=None, point_process=None):
        """Create the vector to record time. If not time_interval specified the 
        NEURON default is used."""
        t = h.Vector()
        if time_interval_recording:
            if point_process: 
                t.record(point_process, 
                         h._ref_t, time_interval_recording)
            else:
                t.record(h._ref_t, time_interval_recording)
        else:
            if point_process: 
                t.record(point_process, 
                         h._ref_t)
            else:
                t.record(h._ref_t)
        self.groups['t'] = t
        return t
    

            
    def get_vector(self, sec, var, group='VecRef'):
        """Return the vec that record the var in a given section
        
        :param sec: Section of interest or the name of the section
        :param var: variable recorded by the vector.
        :return: the vector that record the variable var"""
        
        name = sec
        if hasattr(sec, 'name'):
            name = sec.name()
        for ref in self.refs[group]:
                if ref.sec_name == name:
                    if ref.vecs.has_key(var):
                       return ref.vecs[var]
    def get_time(self):
        """Return the vector time associated with the voltage"""
        return self.manager.groups['t']
    
    def sum_vector(self, vec1, vec2):
        """Sums two vectors with the same length. The vector are 
        converted in numpy array and then summed together 
        
        :param vec1: First addendum
        :param vec2: Second addendum
        :return: The numpy array sum of the two.
        :rtype: Numpy array
        """
        return np.array(vec1) + np.array(vec2)
    
    def get_vectors(self, section_list, var):
        """Return a dictionary containing the vector which record the var. The 
        section name is used as key.
        
        :param section_list: The list of the section which is interested
        :param var: The variable of interest
        :return: The dictionary with section name as key and the vector as the value
        :rtype: dictionary
        """
        vecs = {}
        for sec in section_list:
            vec = self.get_vector(sec, var)
            vecs[sec.name()] = (vec)
        return vecs
            
    
    def get_tree(self, sec):
        """Return the minimal tree of section 
        Using the given section as the last leaf
        
        :param sec: The section that will be used as the last leaf
        :return: The section's tree in a list format"""
        
        tree = []
        tree.append(sec)
        tree = self.__get_parent(sec, tree)
        return tree
    
    def __get_parent(self, sec, tree):
        """Recursive function used to create the tree list of section"""
        sec.push()
        secRef = h.SectionRef()
        if secRef.has_parent():
            parentSeg = secRef.parent()
            parentSec = parentSeg.sec
            tree.append(parentSec)
            tree = self.__get_parent(parentSec, tree)
        h.pop_section()
        return tree
                
    def plot_vecs(self, vecs_dic, x=None, legend=True, figure_num=None, points=False):
        """Plot the vectors with plt
        
        :param vecs_dic: dictionary with section name as k and the vec obj as value
        :param var: Which variable we are plotting.
        :param legend:  If True the legend is plotted
        :param figure_num: in which figure we want to plot the line
        """
        
        if figure_num is not None:
            plt.figure(figure_num)
        else:
            plt.figure()
            
        ax  = plt.subplot(111) # One subplot where to draw everything
         
        for key, vec in vecs_dic.iteritems():
            if points:
                if x is None:
                    plt.plot(self.groups['t'], vec, 'o', label=key)
                elif len(x) != len(vec):
                    plt.plot(vec, 'o', label=key)
                    logging.warning("x and y mismatched. Is the %s wrapped in the right baseref? \
                    Plotted vs it's it own length anyway." %key)
                else:
                    plt.plot(x, vec, 'o', label=key)
            else:
                if x is None:
                    plt.plot(self.groups['t'], vec, label=key)
                elif len(x) != len(vec):
                    plt.plot(vec, label=key)
                    s = "x and y mismatched. Is the %s wrapped in the right baseref?\
                     Plotted vs it's it own length anyway." %key
                    logger.info(s)
                else:
                    plt.plot(x, vec, label=key)
                
            if legend:
                plt.legend(loc=0)

    def create_new_dir(self, prefix="./", root="Sims"):
        """
            Create the directory where to put the simulation
        """
        self.dirRoot = os.path.join(prefix, root)
        
        today = datetime.date.today()
        free = False
        index = 0
        
        dirDate = today.strftime("%d-%m-%Y")
        
        dirComp = os.path.join(self.dirRoot, dirDate)
        dir = os.path.join(dirComp, "Sim_" + str(index))
        while not free :
            if os.path.exists(dir):
                index = index + 1
                simNum = "Sim_" + str(index)
                dir = os.path.join(dirComp, simNum )
            else:
                free = True
                os.makedirs(dir)
        return dir
        
    def sanitized_sec(self, sec_name):
        """Sanitize the neuroML """
        import re
        split = sec_name.split('.') # Getting rid of the Cell name
        sec = ''
        name = ''
        if len(split) == 1:
            sec = split[0]
        elif len(split) == 2: 
            name = split[1]
            m = re.match ('(\w+)\[(\d+)\]', name)
            if m:
                sec = m.group(1) +'_'+ m.group(2)
                
            else:
                sec = name
        return sec
        
    def _save_geom(self, h5file_holder):
        """Store the NeuroML in the geometry table"""
        
        # writing the NeuroML model
        h.define_shape() # We need the 3D points
        
        h.load_file('mview.hoc')
        modelView = h.ModelView(0)
        modelXml = h.ModelViewXML(modelView)
        tmp_file = 'temp.xml'
        modelXml.xportLevel1(tmp_file)
        
        xml_data = ''
        with open(tmp_file, 'r') as f:
            xml_data = f.read()
        geom_group = h5file_holder.createGroup('/', self.geometry_root)
        h5file_holder.createArray(geom_group, self.geometry_node_name, 
                                  [xml_data])
        os.remove(tmp_file)
        
    
    def save_to_hdf(self, filename):
        h5f = tables.openFile(filename, 'w')
        
        # Saving geometry
        
        # Saving the vecRef
        self._save_geom(h5f)
        res = h5f.createGroup('/', self.results_root)
        for group in self.refs:
            self._save_baseRef(self.refs[group], h5f, res)
        h5f.close()
        
    def _save_baseRef(self, baseRefs, h5f_holder, base_group):
        """Save the baseRef in the database"""
        
        target_group = ''
        for baseRef in baseRefs:
            baseRefName = baseRef.group_id
            found = False
            
            for group in h5f_holder.walkGroups('/'):
                if group._v_name == baseRefName:
                    target_group = group
                    found = True
                    break
            
            if not found:
                # Creating the group    
                target_group = h5f_holder.createGroup(base_group, 
                                                      baseRefName)
                
                # Saving the time
                key = target_group._v_name
                x_array = None
                if hasattr(self.groups[key], 'to_python'):    
                    x_array = np.array(self.groups[key])
                else: 
                    x_array = self.groups[key]
                    
                h5f_holder.createArray(target_group, 'x', x_array)
            
            section_name = self.sanitized_sec(baseRef.sec_name)
            detail = ''
            if hasattr(baseRef, 'detail'):
                detail = baseRef.detail
            self.save_node(h5f_holder, target_group, section_name, 
                           baseRef.vecs, detail=detail)
    
    def save_node(self, h5file_holder, group_path, section_name, variables, 
                  detail=''):
        """Save a node to the h5file.
        h5file_holder: The holder of the h5file
        group_path: Where in the hierarchy the leaf has to be saved
        section_name: The name of the section which the variables belong to
        variables: The dictionary of the variable
        """
        found = False
        target_group = None
        for group in h5file_holder.walkGroups(group_path):
            if group._v_name == section_name:
                target_group = group
                found = True
                break
        if not found:
            target_group = h5file_holder.createGroup(group_path, 
                                                     section_name)
        tmp_array = np.array(self.groups['t'])
        for var, vec in variables.iteritems():
            if len (vec) != 0 :
                if hasattr(vec, 'to_python'): # Vector to numpy Array
                    if len(tmp_array) == len(vec):
                        vec = vec.to_python(tmp_array) # Swap in place
                    else:
                        vec =  vec.to_python() # Creating a list
                h5file_holder.createArray(target_group, var, 
                                          vec,
                                          title=detail)
        
    def load_from_hdf(self, filename):
        """Load all the results on the hvf in memory"""
        logger.info( "Loading: %s" %filename)
        self._load_geom(filename)
        self._load_allRef(filename)
        
    
    def _load_geom(self, filename):
        """Load the geometry of the model"""
        
        h5f = tables.openFile(filename)
        node = "/%s/%s" %(self.geometry_root, self.geometry_node_name) 
        geom = h5f.getNode(node)
        xml_data = geom[0]  # get the string.
        
        tmp_file = 'temp.xml'
        f = open(tmp_file, 'w')
        f.write(xml_data)
        f.close()
        
#        import rdxml # This has to go ASAP they fix NEURON install
        h.load_file('celbild.hoc')
        cb = h.CellBuild(0)
        cb.manage.neuroml(tmp_file)
        cb.cexport(1)
        
        os.remove(tmp_file)
    
    def _load_allRef(self, filename):
        """Load the vecref in memory"""
        
         
        h5f = tables.openFile(filename)
        res_group_path = "/%s" %self.results_root
        for group in h5f.iterNodes(where=res_group_path): # Get the type

            # Indipendent var (time for ex)
            x_node = h5f.getNode(group, name='x')
            self.groups[group._v_name] = x_node        
            for group_child in h5f.iterNodes(group): # Get the section name
                if group_child._v_name != 'x':
                    vecs = {}
                    genericRef = None
                    group_ref = group._v_name 
                    sec_name = group_child._v_name
                     
                    if group_ref == 'VecRef':
                        self.groups['t'] = self.groups[group_ref]
                        # Create vecRef
                        nrn_sec = eval('h.' + sec_name)
                        genericRef = VecRef(nrn_sec)
                        
                    else:
                        genericRef = BaseRef() # Creating a genericRef
                        genericRef.sec_name = sec_name
                        
                    genericRef.group_id = group_ref
                    
                    if self.refs.has_key(group_ref):
                        self.refs[group_ref].append(genericRef)
                    else:
                        self.refs[group_ref] = [genericRef]
                    
                    vecs ={}   
                    for node in h5f.iterNodes(where= group_child, 
                                              classname='Array'):
                        
                        vecs[node._v_name] = node
                        genericRef.vecs = vecs
                        genericRef.detail = node._v_title

    def get_distance(self, sections, base_sec):
        """Return a dictionary with section names as keys and 
        distance from base_sec to specified point as value in microns
        
        param:
         secs: list of section which distance need to be calculated
         base_sec: section from where the distance should be measured from."""
         
        distance_sec = {}
        for sec in sections:
            tree = self.get_tree(sec)
            distance = self._get_distance_in_tree(tree)
            distance -= base_sec.L
            distance_sec[sec.name()] = distance
        return distance_sec
    
    def _get_distance_in_tree(self, tree):
        """Calculate the distance in the tree adding the length of all the sections """
        dist = 0
        for sec in tree:
            dist += sec.L
        return dist
    
    def create_3d_grid(self, distance_dic, time, variable_dic):
        """Create the 3 grid (X,Y,Z) uses to plot wireframe or surface 3D 
        
        :param: distance_dic - dictionary with sections' names as key and 
        distance as a value
        :param: time - Time array
        :param: variable_dic - dictionary with sections' names as key and 
        array as value
        
        :rtype: tuple (X,Y,Z) numpy array"""
        # create a grid
        X_time, Y_distance = np.meshgrid(time, distance_dic.values())
        Z_variable = X_time*0
        xn, yn = X_time.shape 
        sec_names = distance_dic.keys()
        # xk iter on distance
        for xk in range(xn):
            # yk on time. It's the reverse of the grid.
            for yk in range(yn):
                var_array = variable_dic[sec_names[xk]]
                Z_variable[xk, yk] =  var_array[yk]
                
        return (X_time, Y_distance, Z_variable)
    
    def plot3D(self, sections, base_sec, var):
        """Wrap up module to plot in three Dimensions the voltage the distance and the time 
        :param: sections - list of section to consider
        :param: base_sec - section used as a starting pooint for the distance
        :param: var - variable to plot """
        
        # I know it's not suggested to import here, but this is pretty new 
        # module for matplotlib and this method is 
        # hardely executed.
        
        from mpl_toolkits.mplot3d import Axes3D
        from matplotlib import cm
        
        dist = self.get_distance(sections, base_sec)
        vecs = self.get_vectors(sections, var)
        X, Y, Z = self.create_3d_grid(dist, self.groups['t'], vecs)
        fig = plt.figure()
        ax = fig.gca(projection='3d')
        surf = ax.plot_surface(X, Y, Z,  rstride=10, cstride=10, cmap=cm.jet,
                               linewidth=0, antialiased=False)
        fig.colorbar(surf, shrink=0.5, aspect=5)
        return fig
        #plt.show()


            
class BaseRef(object):
    """Base Ref class. Subclass it to create your own ref."""
    
    def __init__(self):
        """Initialize the attribute for the baseRef with empty string none"""
        
        self.sec_name = ''
        self.detail = ''
        self.vecs = {}
        self.group_id = self.__class__.__name__
    
    def __str__(self):
        s = "section: %s, detail: %s, vars recorded: %s" %(self.sec_name, 
                                                              self.detail, 
                                                              self.vecs.keys())
        return s
                
class VecRef(BaseRef):
    """Specialized class for HocVectors class to associate one or more vectors with a section
    """
    def __init__(self, sec):
        """Create a vecRef object which map the section name and the 
        recorded vectors.
        
        :param sec: The section which all the vectors belong
        """
        
        BaseRef.__init__(self)
        self.sec_name = sec.name()
        self.sec = sec
        self.pickable = False
        #Dict with all the vecs
        # Key: var Value: Hoc.Vector
        self.vecs = {}
        
        
    def __str__(self):
        return "section: %s, vars recorded: %s" %(self.sec_name, 
                                                  self.vecs.keys())
            
class SynVecRef(BaseRef):
    """Class to track all the synapse quantity of interest"""
    
    def __init__(self, chan_type=None, section_name=None, vecs=None):
        """Create a synVecRef object which map the synapse position and name 
        and the recorded vectors in it.
        
        :param chan_type: The channel in the synapse
        :param section_name: Name of the section where the synapse is
        :param vecs: Dictionary with the synapse vecs
        """
        BaseRef.__init__(self)
        self.detail = chan_type
        self.sec_name = section_name
        self.vecs = vecs

    def __str__(self):
        return "section: %s, chan_type: %s, \
        vars recorded: %s" %(self.sec_name, self.detail, self.vecs.keys())
        


#!/bin/env python

class ModelsTree:
	_treeList = []
	
	def __init__(self, models):
		names = models.get_model_names()
		self._treeList.append(('Recent Additions/Updates', []))
		self._treeList.append(('Local Models', []))
		self._treeList.append(('Marked Models', []))
		self._treeList.append(('Models (' + str(len(names)) + ')', names))
	
	def get_tree(self):
		return self._treeList



# This example comes with NeuronVisio.

# Importing the NeuronVisio
from neuronvisio.controls import Controls
controls = Controls()
# Importing hoc interpreter
from neuron import h

# simple model

soma = h.Section(name='soma')
dend1 = h.Section(name='dend1')
dend2 = h.Section(name='dend2')

soma.diam = 10
dend1.diam = 3
dend2.diam = 3
soma.L = 50
dend1.L = 100
dend2.L = 100
  
dend1.connect(soma)
dend2.connect(soma)

# biophysics

for sec in h.allsec(): 
     sec.Ra = 100      
     sec.cm = 1         

soma.insert('hh')
dend1.insert('pas')
dend2.insert('pas')

# --------------------- Instrumentation ---------------------
# synaptic input
syn = h.AlphaSynapse(0.5, sec=soma)
syn.onset = 0.5
syn.gmax = 0.05
syn.e = 0



#"""
#NeuronVisio is a Graphical User Interface for NEURON simulator enviroment
#
#Copyright (c) 2009, Michele Mattioni
#All rights reserved.
#
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#"""

__authors__ = ['Michele Mattioni <mattioni@ebi.ac.uk>', 
               'Uri Cohen <uri.cohen@alice.nc.huji.ac.il>']
__version__ = '0.7.2'


import os
# this add the commit of to the software version when run from a git repo
try:
    import git
    if git.__version__ == '0.3.1':
        src_root = os.path.dirname(__path__[0]) # Getting the root of the module
        root = os.path.join(src_root, '../')
        if git.repo.fun.is_git_dir(os.path.join(root, ".git")):
            r = git.Repo(root)
            git_commit = r.head.commit.hexsha
            __version__ = __version__ + ' : ' + git_commit
        raise ImportError
except ImportError:
    pass



"""
colorbutton.py

A tool button that enables the user to select colors using a color dialog.

Copyright (C) 2007 David Boddie <david@boddie.org.uk>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *

class ColorButton(QToolButton):

    __pyqtSignals__ = ("colorChanged(QColor)")
    
    def __init__(self, parent = None):
    
        QToolButton.__init__(self, parent)
        self.connect(self, SIGNAL("clicked()"), self.chooseColor)
        self._color = QColor()
    
    def chooseColor(self):
    
        rgba, valid = QColorDialog.getRgba(self._color.rgba(), self.parentWidget())
        if valid:
            color = QColor.fromRgba(rgba)
            self.setColor(color)
    
    def color(self):
    
        return self._color
    
    @pyqtSignature("QColor")
    def setColor(self, color):
    
        if color != self._color:
            self._color = color
            self.emit(SIGNAL("colorChanged(QColor)"), self._color)
            pixmap = QPixmap(16, 16)
            pixmap.fill(color)
            self.setIcon(QIcon(pixmap))
    
    color = pyqtProperty("QColor", color, setColor)



#!/usr/bin/env python
# Author Michele Mattioni
# Fri Apr  9 11:35:29 BST 2010

"""Main script to start Neuronvisio"""

from neuronvisio.controls import Controls
import sys


if __name__ == '__main__':
    
    controls = Controls()
    if len(sys.argv) == 2:
        controls.load_hdf(sys.argv[1])
    
         


#!/bin/env python
import types
import string
import re
import os
import xml.etree.ElementTree
import urllib
import zipfile
import logging 
import subprocess

logger = logging.getLogger(__name__)

HTML_BODY="<body bgcolor='#999999' text='#FFFFFF'>"
HTML_TABLE="<table bgcolor='#FFFFFF' width='100%%' cellspacing='2' cellpadding='2' border='1'>"

class Model(object):
    # Fields
    _model = None
    
    # Constants
    _MODEL_PROPERTIES = [('model_type', 0), ('model_concepts', 0),
                        ('transmitters', 0),('genes', 0), ('receptors', 0), ('brain_regions', 0),
                        ('channels', 0), ('cell_types', 0), ('gap_junctions', 0), 
                        ('implementers', 0),
                        ('simulation_environment', 0)]
    _MODEL_PAGE_LAYOUT = """%(short_name)s|<b>%(title)s</b>
        <b>Description</b>: %(description)s
        <b>Reference</b>: %(reference)s
        <a href='%(url)s'>ModelDB Page</a>|<a href='%(citations)s'>Citations Query</a>|Model #%(model_id)s"""
    _README_NOT_FOUND = "A readme file was not found for this model, try online at ModelDB"
    
    # Initialization
    def __init__(self, model):
        self._model = model

    # Public data accessors
    # Get a mapping of non-empty model properties
    def get_properties(self):
        for (k, s) in self._MODEL_PROPERTIES:
            t=self._model[k]
            t=t.strip()
            if len(t) == 0: # Show only non-empty properties
                continue
            k=k.replace('_', ' ')
            k=k.title()
            yield (k, t)
            
    def get_name(self):
        return self._model['short_name']

    def get_title(self):
        return self._model['title']

    def get_description(self):
        return self._model['description']

    def get_reference(self):
        return self._model['reference']

    def get_url(self):
        return self._model['url']

    def get_id(self):
        return int(self._model['model_id'])

    def get_authors(self):
        return self._model['short_authors']

    def get_year(self):
        return self._model['year']

    # Get the model readme in text
    def get_readme_text(self):
        if self._model.has_key('readme') == False:
            return None
        return self._model['readme']

    # Get the model readme in HTML
    def get_readme_html(self):
        readme = self.get_readme_text()
        if readme == "":
            return self._create_readme_tab(self._README_NOT_FOUND)
        return self._create_readme_tab(readme)

    # Get the model overview
    def get_overview(self):
        return self._create_overview_tab()
    
    # Public operations and manipulations
    # Check if model exists locally
    def exists_locally(self):
        return os.path.isdir(self.get_dir())

    # Open model directory for browsing
    def browse(self):
        if self.exists_locally():
            modelName = self.get_name()
            logger.info("Opening '" + self.get_dir()+"'.")
            self._start_file(self.get_dir())
        else:
            logger.info("Model does not exists locally")

    # Download model file from network
    def download_model(self):
        modelId = self.get_id()
        if os.path.isdir('Models')==False:
            os.mkdir('Models')        
        model_zipped= ('%s.zip') %modelId
        zipFile = os.path.join('Models', model_zipped)
        modelDir = self.get_dir()
        if not os.path.isfile(zipFile):
            if self._model['zip_url']=="":
                logger.info("No zip URL found for '" + self.get_name() + "'")
                return
            else:
                logger.info("Downloading model for '" + self.get_name() + "'")
                self._download_file(self._model['zip_url'], zipFile)
                logger.info("Download complete.")
                # TODO: open model data in tab
                # TODO: recolor, self.tree.SetItemColor(item, wx.Colour(100,10,255))
        else:
            logger.info("Model for '" + self.get_name() + "' already downloaded")
        if not os.path.isdir(modelDir):
            self._extract_model(zipFile, modelDir)
        else:
            logger.info("Model for '" + self.get_name() + "' already extracted")
        return modelDir

    # Get model directory
    def get_dir(self):
        modelId = self.get_id()
        dirName = os.path.join('Models', str(modelId))
        return dirName
    
    def get_tooltip(self):
        "Return the tooltip for the model."
        tooltip = ''
        if self.exists_locally():
            tooltip = 'Model %s downloaded in %s' %(self.get_id(), 
                                                    self.get_dir())
        else: 
            tooltip = 'Model %s has not been downloaded' %self.get_id()
        return tooltip

    # Private implementation methods
    # Download model file from the network
    def _download_file(self, url, filename):
        logger.info("Creating " + filename)
        try:
            s = urllib.urlopen(url)
            f = open(filename, "wb")
            f.write(s.read())
            f.close()
        except Exception as e:
            logger.info("Error downloading file: "+e.message)
            return
        logger.info("Done.")

    # Extract model zip file
    # ModelDB zip files contain trailing garbage which should be removed
    # See 'ZIP end of central directory record' in http://en.wikipedia.org/wiki/ZIP_%28file_format%29
    def _extract_model(self, zipFile, modelDir):
        logger.info("Extracting '" + zipFile + "' into " + modelDir)
        try:
            f = open(zipFile, 'r+b')
            data = f.read()
            pos = data.find('\x50\x4b\x05\x06') # End of central directory signature
            if (pos > 0):
                logger.info("Trancating file at location " + str(pos + 22)+ ".")
                f.seek(pos + 22)                # size of 'ZIP end of central directory record'
                f.truncate()
                f.close()
            zip = zipfile.ZipFile(zipFile, 'r')
            dirs = {}
            for file in zip.namelist():
                dir = file[0:file.find('/')]
                if dirs.has_key(dir)==False:
                    dirs[dir]=0
                else:
                    dirs[dir]=dirs[dir]+1
                zip.extract(file, 'Models')
            zip.close()
            if len(dirs.keys()) == 1:
                os.rename('Models/' + dirs.keys()[0], modelDir)
            else:
                os.mkdir(modelDir)
                for d in dirs:
                    os.rename('Models/' + d, modelDir + d)
        except Exception as e:
            logger.info("Error extracting file: "+str(e.message))
        logger.info("Done.")

    # Create the HTML of the readme tab
    def _create_readme_tab(self, readme):
        html = "<html>"
        html = html +HTML_BODY
        html = html + "<p>" + HTML_TABLE
        html = html + "<tr>" + readme + "</tr>"
        html = html + "</table></p>"
        html = html + "</body>"
        html = html + "</html>"
        return html

    # Create the HTML of the overview tab
    def _create_overview_tab(self):
        html = "<html>"
        html = html +HTML_BODY
        html = html + "<table border='0'><tr><td>"
        html = html + "<h1>Model Information</h1>"
        html = html + "<p>" + self._generate_table(self._MODEL_PAGE_LAYOUT, 3) + "</p>"
        html = html + "</td><td>"
        html = html + "<td><h1>Model Properties</h1>"
        html = html + "<p>" + HTML_TABLE
        for k,s in self.get_properties():
            html = html + "<tr><td><strong>" + k + "</strong></td><td>" + s + "</td></tr>"
        html = html + "</table></p>"
        html = html + "</td></tr></table>"
        html = html + "</body>"
        html = html + "</html>"
        return html

    # Create an HTML table from the given table format
    def _generate_table(self, format, cols):
        properties = self._model
        result=HTML_TABLE
        lines=format.split('\n')
        for l in lines:
            a=''
            i=0
            cells=l.split('|')
            v=cols-len(cells)+1
            for c in cells:
                i=i+1
                if (i == len(cells)):
                    a=a+'<td colspan="' + str(v) + '">'+c%properties+'</td>'
                else:
                    a=a+'<td>'+c%properties+'</td>'
            result = result + '<tr>' + a + '</tr>'
        result = result + "</table>"
        return result

    # Open a directory for browsing its content (implementation taken from SO)
    # http://stackoverflow.com/questions/2878712/make-os-open-directory-in-python
    def _start_file(self, filename):
        try:
            # Implementation for Windows
            f=filename.replace('/', '\\')
            os.startfile(f)
        except:
            # Implementation for Linux
            subprocess.Popen(['xdg-open', filename])

#---------------------------------------------------------------------------
class Models():
    # Fields
    _log = None
    _name_re = None   
    _modelList = []
    _modelTree = {}

    # public methods
    # Initialize class, assuming the XML model DB is in the current directory
    def __init__(self):
        self._name_re = re.compile("^\s*(.*)\(((.*)(\d{4}).*)\)")
        path = os.path.split(os.path.abspath(__file__))[0] # base absolute dir 
        self._modelList = self._generate_models_list(os.path.join(path, 'ModelDB.xml'))
        self._modelTree = self._generate_models_tree(self._modelList)        

    # Get all model names
    def get_model_names(self):
        return self._modelTree.keys()

    # Check if specific model exists
    def has_model(self, modelName):
        return self._modelTree.has_key(modelName)

    # Get specific model
    def get_model(self, modelName):
        return Model(self._modelTree[modelName])

    # Get the value of specific model field
    def get_field(self, model, field):
        return self._modelTree[model][field]
    
    """ Returns whether a model contains the search keyword or not. """
    def search(self, name, keyword):
        return self._modelTree[name]['searchable'].find(keyword.lower()) >= 0
       
    # private methods
    # Generate the models list from the input XML
    def _generate_models_list(self, data):
        models=xml.etree.ElementTree.XML(open(data).read())
        list=[]
        for m in models:
            d={}
            for c in m._children:
                if (len(c._children)>0):
                    d[c.tag]=map((lambda a:a.text), c._children)
                else:
                    d[c.tag]=c.text
            list.append(d)
        return list

    # Generate the models tree, moving each item into canonical form
    def _generate_models_tree(self, list):
        tree={}
        for m in list:
            search=''
            for s in m.keys():
                search = search + "|" + self._to_text(m[s])
            m['searchable']=search.lower()
            name=m['name']
            [t, s, a, y]=self._get_title_authors_year_short_name(name)
            m['short_name']=s
            m['title']=t
            m['short_authors']=a
            m['year']=y
            c=self._to_text(m['citations'])
            if len(c)>0:
                m['citations']=self._MODELDB_BASE_URL + c
            for k in m.keys():
                m[k]=self._to_text(m[k])
            tree[s]=m
        return tree

    # Parse model short name into arguments
    def _get_title_authors_year_short_name(self, name):
        m=self._name_re.match(name)
        if m:
            return m.groups()
        else:
            raise Exception("no match in " + name)

    # Handle string, none and list values and change them to be stripped string
    def _to_text(self, t):
        if (type(t) == types.ListType):
            t=string.join(t, ',')
        if (type(t) == types.NoneType):
            t=''
        t=t.strip()
        return t

    # Constants
    _MODELDB_BASE_URL = 'http://senselab.med.yale.edu/modeldb/'



"""
This example comes with NeuronVisio.
Cell model taken from the paper:

NEURON and Python
Hines et al.
Frontiers in Neuroinformatics 
(2009)
DOI 10.3389/neuro.11/001.2009 

"""

from itertools import chain

# Importing Neuronvisio
from neuronvisio.controls import Controls
controls = Controls()

# Importing the hoc interpreter
from neuron import h

# topology
soma = h.Section(name='soma')
apical = h.Section(name='apical')
basilar = h.Section(name='basilar')
axon = h.Section(name='axon')

apical.connect(soma, 1, 0)
basilar.connect(soma , 0, 0)
axon.connect(soma, 0, 0)

# geometry      
soma.L = 30    
soma.nseg = 1 
soma.diam = 30
                                                                          
apical.L = 600                       
apical.nseg = 23                     
apical.diam = 1                      
                                     
basilar.L = 200
basilar.nseg = 5                  
basilar.diam = 2
                
                                       
axon.L = 1000                         
axon.nseg = 37                       
axon.diam = 1                        
                                     
# biophysics
for sec in h.allsec(): 
     sec.Ra = 100      
     sec.cm = 1         

soma.insert('hh')                  
                                   
                                   
apical.insert('pas')           
                                   
basilar.insert('pas')                   
                                   
for seg in chain (apical, basilar ):
     seg.pas.g = 0.0002             
     seg.pas.e = -65                

axon.insert('hh')                  


# --------------------- Instrumentation ---------------------
# synaptic input
syn = h.AlphaSynapse(0.5, sec=soma)
syn.onset = 0.5
syn.gmax = 0.05
syn.e = 0



#!/bin/env python
import urllib2
import time
import logging
import xml.dom.minidom
import re
import types
import codecs
import os.path
from BeautifulSoup import BeautifulSoup

logger = logging.getLogger(__name__)

# This class allows updating a ModelDB local store from the online site.
# 
# It also offer a stand-alone main() which update the ModelDB.xml file
# from current directory.
class ModelDBUpdater:
	# Constants
	_BASE_URL = 'http://senselab.med.yale.edu/modeldb/'
	_START_URLS = ["http://senselab.med.yale.edu/modeldb/ModelList.asp?id=1882"]
	_DOWNLOAD_DELAY = 1
	_ID_REGEX = re.compile("ShowModel\.asp\?model=(\d*)")
	_NAME_REGEX = re.compile("^\s*(.*)\(((.*)(\d{4}).*)\)")
	
	# Private variables
	_last_download_time = 0
	_xml_file = ""
	_existing_items = {}
	_dom = None

	# Initialize an updater for a file with local ModelDB content
	def __init__(self, xml_file):
		self._xml_file = xml_file
		# Create the file if it does not exist
		if os.path.isfile(xml_file)==False:
			f=codecs.open(xml_file, "w", encoding='utf-8')
			f.write('<?xml version="1.0" ?><items/>'.decode('utf-8'))
			f.close()
		# Read the file into dom
		logger.info("Reading %s..."%xml_file)
		self._dom = xml.dom.minidom.parse(xml_file)
		# Create a list of available id-s
		items=self._dom.childNodes[0]
		ids=items.getElementsByTagName('model_id')
		for i in ids:
			id = i.childNodes[0].toxml().strip()
			self._existing_items[id] = True
		logger.info("Got %d models locally"%len(self._existing_items.keys()))

	# Update models XML file from online content
	def update(self):
		# Get online items and compare them to the local items
		online = self.get_online_items()
		logger.info("Got %d online items"%len(online))	
		new_ones = self.get_new_items(online)
		logger.info("Got %d new items"%len(new_ones))
		# Add an item to the dom for each new item
		for i in new_ones.keys():
			u=new_ones[i]['url']
			v=self.parse_item(u)
			new_item = self._dom.createElement('item')
			for k in v.keys():
				name = self._dom.createElement(k)
				if type(v[k]) == types.ListType:
					# Create a node named after the key with childrens named 'value' for each value
					for val in v[k]:
						value = self._dom.createElement('value')
						value.appendChild(self._dom.createTextNode(self._get_text(val)))
						name.appendChild(value)
					# "The most common shortcoming of BeautifulStoneSoup is that it  doesn't know about
					# self-closing tags" so we need to add empty content in them
					if len(v[k])==0:
						name.appendChild(self._dom.createTextNode(u""))
				else:
					name.appendChild(self._dom.createTextNode(self._get_text(v[k])))
				new_item.appendChild(name)
			self._dom.childNodes[0].appendChild(new_item)
		self.update_xml()
		return new_ones

	# Compare existing items to online ones
	def get_new_items(self, online_items):
		online_ids = online_items.keys()
		new_items = {}
		for i in online_items:
			if self._existing_items.has_key(i)==False:
				new_items[i]=online_items[i]
				logger.info("New item %d %s"%(int(i), new_items[i]['name']))
		return new_items

	# Find all online items 
	def get_online_items(self):
		results = {}
		for u in self._START_URLS:
			list_data = self._get_url(u)
			soup = BeautifulSoup(list_data)
			for i in soup('tr'):
				if i.td.a==None or i.td.a.string==None:
					logger.warn("Ignoring empty item: %s"%i)
				else:
					# Filter only idems which have a valid id and name
					# This prevents collecting place-holders for yet-to-be-published items
					m=self._ID_REGEX.match(i.td.a['href'])
					n=self._NAME_REGEX.match(i.td.a.string)
					if not m:
						logger.warn("Ignoring bad model link: '%s'"%i.td.a['href'])
					elif not n:
						logger.warn("Ignoring bad model name: '%s'"%i.td.a.string)
					else:
						id = m.groups()[0]
						url = self._BASE_URL + i.td.a['href']
						name = i.td.a.string
						map = {'url':url, 'name':name, 'id':id}
						results[id] = map
		return results

	# Write DOC back into XML file
	def update_xml(self):
	    f = codecs.open(self._xml_file, "w", encoding='utf-8')
	    xml=self._dom.toxml(encoding="utf-8")
	    soup = BeautifulSoup(xml)
	    f.write(soup.prettify().decode('utf-8')) 
	    f.close()

	# Parse a ModelDB model page and extract an item dictionary	
	def parse_item(self, url):
		data=self._get_url(url)

		# Extract main items from the page
		soup = BeautifulSoup(data)
		table1 = soup.find('table', {'id': 'Table1'})
		table2 = soup.find('table', {'id': 'Table2'})
		table3 = soup.find('table', {'id': 'Table3'})
		if table1 == None or table2 == None or table3 == None:
			exit("Could not find a mandatory table")
		rows = table1.findAll('tr', {}, False)
		if rows == None:
			exit("Could not find model details")
		props = table2.findAll('tr', {}, False)
		if props == None:
			exit("Could not find model properties")
		files = table3.findAll('tr', {}, False)
		if files == None:
			exit("Could not find model files")
		
		item={}		
		item['url']=url.encode('utf-8')
		
		# Model details
		item['name'] = rows[0].th.string
		if item['name'] == None:
			exit("Could not find mandatory field: name")
		item['model_id'] = rows[1].th.contents[2]
		if item['name'] == None:
			exit("Could not find mandatory field: name")
		item['description'] = self._html_to_text(rows[2].td.contents)
		if item['description'] == None:
			exit("Could not find mandatory field: description")
		item['reference'] = self._html_to_text(rows[2].td.contents[3:])
		if rows[2].td.em != None and rows[2].td.em.a != None:
			item['article'] = rows[2].td.em.a['href']
		if rows[2].td.small != None and rows[2].td.small.a != None:
			item['pubmed'] = rows[2].td.small.a['href']
		item['citations'] = rows[3].td.a['href']

		# Model properties
		item['model_type'] = self._list_to_text(props[0].findAll('td', {}, False)[1]('a'))
		item['brain_regions'] = self._list_to_text(props[1].findAll('td', {}, False)[1]('a'))
		item['cell_types'] = self._list_to_text(props[2].findAll('td', {}, False)[1]('a'))
		item['channels'] = self._list_to_text(props[3].findAll('td', {}, False)[1]('a'))
		item['gap_junctions'] = self._list_to_text(props[4].findAll('td', {}, False)[1]('a'))
		item['receptors'] = self._list_to_text(props[5].findAll('td', {}, False)[1]('a'))
		item['genes'] = self._list_to_text(props[6].findAll('td', {}, False)[1]('a'))
		item['transmitters'] = self._list_to_text(props[7].findAll('td', {}, False)[1]('a'))
		item['simulation_environment'] = self._list_to_text(props[8].findAll('td', {}, False)[1]('a'))
		if item['simulation_environment'] == None:
			exit("Could not find mandatory field: simulation_environment")
		item['model_concepts'] = self._list_to_text(props[9].findAll('td', {}, False)[1]('a'))
		item['implementers'] = self._list_to_text(props[10].findAll('td', {}, False)[1]('a'))

		# Model files
		links = files[0].td.div.findAll('a', None, False)
		zip_url=links[0]['href']
		item['zip_url']=zip_url
		if item['zip_url'] == None:
			exit("Could not find mandatory field: zip_url")
		readme = files[1].findAll('td', None, False)[1]
		item['readme']=readme.renderContents().decode('utf-8')
		if item['readme'] == None:
			exit("Could not find mandatory field: readme")
		return item

	# Download data from URL with appropriate delays
	def _get_url(self, url):
		# Delay the download by some time, if needed
		time_since_download = time.time() - self._last_download_time
		if time_since_download  < self._DOWNLOAD_DELAY:
			s = self._DOWNLOAD_DELAY - time_since_download
			logger.debug("Sleeping for %0.2f seconds"%s)
			time.sleep(s)
			
		# Perform the actual download
		logger.info("Reading %s"%url)
		data = urllib2.urlopen(url).read()
		self._last_download_time = time.time()		
		return data

	# Extract text from a contents list, concatanating the text of the items into a single string
	def _html_to_text(self, h):
		t=u""
		for i in h:
			if type(i).__name__=='NavigableString':
				t=t+i
			elif i.string != None:
				t=t+i.string
			else:
				logging.debug("No text extracted from %s"%str(i))
		return t

	# Extract text from a list, returning a list of strings
	def _list_to_text(self, l):
		v=[]
		for i in l:
			v.append(self._html_to_text(i))
		return v
	
	# Extract text from BeautifulStoneSoup node into a string
	def _get_text(self, node):
		if type(node).__name__ == 'NavigableString':
			return node.string
		elif type(node).__name__ == 'NavigableUnicodeString':
			return node.string
		elif type(node) == types.StringType:
			return unicode(node, 'utf-8')
		elif type(node) == types.UnicodeType:
			return node
		elif node == None:
			# See the comment above about the shortcoming of BeautifulStoneSoup
			return u""
		return node.renderContents()

# A stand-along main which updates ModelDB.xml in current directory
def main():
	# Setup logging to stdout
	import sys
	h = logging.StreamHandler(sys.stderr)
	h.setLevel(logging.DEBUG)
	logging.getLogger().addHandler(h)
	logging.getLogger().setLevel(logging.INFO)

	# Update models	
	updater = ModelDBUpdater('ModelDB.xml')
	updater.update()
	
if __name__ == '__main__':
    main()


# * Copyright (C) Wed Jan  6 10:17:03 GMT 2010 - Michele Mattioni:
# *  
# * This file is part of NeuronVisio
# * 
# * NeuronVisio is free software: you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation, either version 3 of the License, or
# * (at your option) any later version.
#
# * NeuronVisio is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
#
# * You should have received a copy of the GNU General Public License
# * along with NeuronVisio.  If not, see <http://www.gnu.org/licenses/>.

""":synopsis: 3D Visual operations

Contain all the 3D operations.
"""


import os
os.environ['ETS_TOOLKIT'] = 'qt4'
import logging
logger = logging.getLogger(__name__)

from PyQt4 import QtGui, QtCore, uic
from enthought.traits.api import HasTraits, Instance, on_trait_change, \
    Int, Dict
from enthought.traits.ui.api import View, Item
from enthought.mayavi.core.ui.api import MayaviScene, MlabSceneModel, \
        SceneEditor

import numpy as np
from enthought.mayavi import mlab
#mlab.options.backend = 'envisage'



from bisect import bisect_left

from neuron import h


################################################################################
#The actual visualization
class Visualization(HasTraits):
    scene = Instance(MlabSceneModel, ())

    @on_trait_change('scene.activated')
    def update_plot(self):
        # This function is called when the view is opened. We don't
        # populate the scene when the view is not yet open, as some 
        # VTK features require a GLContext.

        # We can do normal mlab calls on the embedded scene.
#        self.scene.mlab.test_points3d()
        pass

    # the layout of the dialog screated
    view = View(Item('scene', editor=SceneEditor(scene_class=MayaviScene),
                     height=400, width=500, show_label=False),
                resizable=True # We need this to resize with the parent widget
                )


################################################################################
# The QWidget containing the visualization, this is pure PyQt4 code.
class MayaviQWidget(QtGui.QWidget):
    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        layout = QtGui.QVBoxLayout(self)
        layout.setMargin(0)
        layout.setSpacing(0)
        self.visualization = Visualization()

        # If you want to debug, beware that you need to remove the Qt
        # input hook.
        #QtCore.pyqtRemoveInputHook()
        #import pdb ; pdb.set_trace()
        #QtCore.pyqtRestoreInputHook()

        # The edit_traits call will generate the widget to embed.
        self.ui = self.visualization.edit_traits(parent=self, 
                                                 kind='subpanel').control
        layout.addWidget(self.ui)
        self.ui.setParent(self)



class Visio(object):
    
    def __init__(self, sec_info_label, manager):       
        
        # Needed when user pick the cylinder from visio and 
        # we need to get the section
        self.cyl2sec = {}
        
        # Needed to update the value of a cyl bound to a section
        self.sec2cyl = {}
        
        self.seg2id = {}
        self.sec2coords = {}
        self.connections = []
        self.n3dpoints_per_sec = {}
           
        self.selected_cyl = None # Used for storing the cyl when picked
        self.sec_info_label = sec_info_label # Info for the selected sec
        self.manager = manager
        
        container = QtGui.QWidget()
        container.setWindowTitle("Neuronvisio 3D")
        
        self.mayavi = MayaviQWidget(container)
        layout = QtGui.QVBoxLayout(container)
        layout.addWidget(self.mayavi)
                
        # Tell visual to use this as the viewer.
        #visual.set_viewer(self.mayavi.visualization.scene)
        
        # binding to hide event.
        container.connect(container, QtCore.SIGNAL('closeEvent()'), 
                               self.closeEvent)
        
        container.show()
        
        self.container = container
        
        # Connecting the picker.
        figure = self.mayavi.visualization.scene.mlab.gcf()
        self.outline = None
        self.picker = figure.on_mouse_pick(self.picker_callback, type='cell')
        
        # ScalarBar and time_point
        self.colorbar = None
        self.timelabel = None
    

    def picker_callback(self, picker):
        """ Picker callback: this get called when on pick events.
        """
        # Outline
        
        logger.debug( picker.pick_position)
        
        bounds = self.cyl2sec.keys()
        for bound in bounds:
            x_b, y_b, z_b = bound[0], bound[1], bound[2]
            if bisect_left(x_b, self.picker.pick_position[0]) == 1:
              if bisect_left(y_b, self.picker.pick_position[1]) == 1:
                  if bisect_left(z_b, self.picker.pick_position[2]) == 1:
                      selected_sec = self.cyl2sec[bound]
                      logger.info("Selected Section: %s" %selected_sec.name())
                      
                      self.update_sections_info(selected_sec)
                      
                      info = self.get_sec_info(selected_sec)
                      self.sec_info_label.setText(info)
                      # Creating the list with selected secs.
                      # Only one passing the name
                      selected_secs_name = []
                      # only one sec selected.
                      if hasattr(selected_sec, 'name'):
                          selected_secs_name = selected_sec.name()
                      # More than one.
                      # This happens when we can't select only one 
                      # sec with the box approach...
                      else:
                          for sec in selected_sec:
                              selected_secs_name.append(sec.name())
                      new_scalar = self.get_selection_scalar(selected_secs_name)
                              
                      self.redraw_color(new_scalar, 'v')
                      

    
    def update_sections_info(self, sections):
        """Updating the info tab for the given section"""
        # Check if it's only one and retrieve info
        if hasattr(sections, 'name'):
            selected_secs_name = sections.name()
            info = self.get_sec_info(sections)
            self.sec_info_label.setText(info)
        # Print the list on the tab.
        else: 
            info = "Selected sections: %s" %sections
            self.sec_info_label.setText(info)
    
    def get_selection_scalar(self, selected_secs):
        """ Return a scalar array with zero everywhere but one 
        for the selected sections
        
        :param: selected_secs - list with the names of the selected sections 
        
        """
        
        new_scalar = []
        for sec in h.allsec():
            if sec.name() in selected_secs:
                sec_scalar = self.build_sec_scalar(sec, 1.)
            else:
                sec_scalar = self.build_sec_scalar(sec, 0.)
            new_scalar.extend(sec_scalar) 

        return np.array(new_scalar)
    
    def get_sec_info(self, section):
        """Get the info of the given section"""
        
        info = "<b>Section Name:</b> %s<br/>" %section.name()
        info += "<b>Length [um]:</b> %f<br/>" % section.L
        info += "<b>Diameter [um]:</b> %f<br/>" % section.diam
        info += "<b>Membrane Capacitance:</b> %f<br/>" % section.cm
        info += "<b>Axial Resistance :</b> %f<br/>" % section.Ra
        info += "<b>Number of Segments:</b> %f<br/>" % section.nseg
        mechs = []
        for seg in section:
            for mech in seg:
                mechs.append(mech.name())
        mechs = set(mechs) # Excluding the repeating ones
        mech_info = "<b>Mechanisms in the section</b><ul>"
        for mech_name in mechs:
            s = "<li> %s </li>" % mech_name
            mech_info += s
        mech_info += "</ul>"
        info += mech_info
        return info
                
            

    
    def closeEvent(self):
        """Just hide the window to not loose the mayavi hook"""
        self.container.hide()
        
    def draw_model(self):
        """Draw the model.
        Params:
        controls - the main gui obj."""
        
        # Draw the new one
        h.define_shape()
        num_sections = 0

        # Disable the render. Faster drawing.
        self.mayavi.visualization.scene.disable_render = True
        

        x,y,z,d = [], [], [], []
        voltage = []
        connections = []
        for sec in h.allsec():
            x_sec, y_sec, z_sec, d_sec = self.retrieve_coordinate(sec)
            self.sec2coords[sec.name()] = [x_sec, y_sec, z_sec]
            # Store the section. later.
            radius = sec.diam/2.
            sec_coords_bound = ((x_sec.min(), x_sec.max()), 
                                (y_sec.min() - radius, 
                                 y_sec.max() + radius), 
                                (z_sec.min() - radius, 
                                 z_sec.max() + radius))
            self.cyl2sec[sec_coords_bound] = sec 
            self.sec2cyl[sec] = sec_coords_bound
            
            
            for i,xi in enumerate(x_sec):
                x.append(x_sec[i])
                y.append(y_sec[i])
                z.append(z_sec[i])
                d.append(d_sec[i])
                indx_geom_seg = len(x) -1
                
                if len(x) > 1 and i > 0:
                    connections.append([indx_geom_seg, indx_geom_seg-1])
            
                    
        self.edges  = connections
        self.x = x
        self.y = y
        self.z = z
        
        # Mayavi pipeline        
        d = np.array(d) # Transforming for easy division
        
        self.draw_mayavi(x, y, z, d, self.edges)
        
    def get_var_data(self, var, time_point=0):
        """Retrieve the value of the `var` for the `time_point`.
        Prameters:
        var - variable to retrieve
        time_point - point in the simulation"""
        
        var_scalar = []
        for sec in h.allsec():
            var_value = 0
            if self.manager.refs.has_key('VecRef'):
                for vecRef in self.manager.refs['VecRef']:
                    if vecRef.sec.name() == sec.name():
                        if vecRef.vecs.has_key(var):
                            vec = vecRef.vecs[var]
                            var_value = vec[time_point]
            sec_scalar = self.build_sec_scalar(sec, var_value)
            var_scalar.extend(sec_scalar)
                
                    
                        
        
        if len(var_scalar) == 0:
            logger.debug( "Var scalar 0 length. Var: %s point_time: %s" %(var, 
                                                                  time_point))
        return np.array(var_scalar)

    def build_sec_scalar(self, sec, var_value):
        
        sec.push()
        npoints = self.n3dpoints_per_sec[sec.name()]
        sec_scalar = np.repeat(var_value, npoints)
        h.pop_section()
        return sec_scalar

        
    def draw_mayavi(self, x, y, z, d, edges):
        "Draw the surface the first time"
        
        # rendering disabled
        self.mayavi.visualization.scene.disable_render = True
        
        points = mlab.pipeline.scalar_scatter(x, y, z, d/2.0)
        dataset = points.mlab_source.dataset
        dataset.point_data.get_array(0).name = 'diameter'
        dataset.lines = np.vstack(edges)
        dataset.point_data.update()
        self.dataset = dataset

        # The tube
        src = mlab.pipeline.set_active_attribute(points, point_scalars='diameter')
        stripper = mlab.pipeline.stripper(src)
        tube = mlab.pipeline.tube(stripper, tube_sides = 6, tube_radius = 1)
        tube.filter.capping = True
#        tube.filter.use_default_normal = False
        tube.filter.vary_radius = 'vary_radius_by_absolute_scalar'
        self.tube = tube
        

        # Setting the voltage
        # Making room for the voltage
        v = []
        for sec in h.allsec():
            sec.push()
            v.extend(np.repeat(0.0, h.n3d()))
            h.pop_section()
        
        v = np.array(v)
        self.draw_surface(v, 'v')
        
        # ReEnable the rendering
        self.mayavi.visualization.scene.disable_render = False

        
    def draw_surface(self, scalar, scalar_name):
        
        self.tube.children= [] # Removing the old ones 
        
        scalar = self.get_var_data('v', 0)
        array_id = self.dataset.point_data.add_array(scalar)
        self.dataset.point_data.get_array(array_id).name = scalar_name
        self.dataset.point_data.update()
        src2 = mlab.pipeline.set_active_attribute(self.tube, 
                                                  point_scalars=scalar_name)
        self.surf = mlab.pipeline.surface(src2, colormap='blue-red')
        
    
    def show_variable_timecourse(self, var, time_point, 
                                 start_value, end_value):
        """Show an animation of all the section that have 
        the recorded variable among time"""
        
        
        # Getting the new scalar
        new_scalar = self.get_var_data(var, time_point)
        
        d = self.dataset.point_data.get_array('diameter')
        if len(d) != len(new_scalar):
            message = "ERROR! MISMATCH on the Vector Length. \
            If you assign the new vectors it will not work \
            Diameter length: %s New Scalar length: %s var: %s" %(len(d),
                                                                 len(new_scalar),
                                                                 var)
            logger.error(message)
        # ReEnable the rendering
        self.mayavi.visualization.scene.disable_render = True
        
        self.redraw_color(new_scalar, var)
        

        if not self.colorbar:
            self.colorbar = mlab.colorbar(orientation='vertical')
            self.timelabel = mlab.text(0.05, 
                                       0.05, 
                                       str(time_point), 
                                       width=0.05
                                       )
        
        self.colorbar.data_range = [start_value, end_value]
        time = self.manager.groups['t'][time_point]
        self.timelabel.text = str(round(time, 3))
        
        self.mayavi.visualization.scene.disable_render = False

    
    def redraw_color(self, new_scalar, var):
        """Redraw the tubes with the new scalar.
        new_scalar -- the scalar used to color the tubes
        var - the name of the variable used"""
        #self.tube.children= [] # Removing the old ones
        
        array_id = self.dataset.point_data.add_array(new_scalar)
        self.dataset.point_data.get_array(array_id).name = var
        self.dataset.point_data.update()
        
        # Updating the dataset
        ms = self.surf.mlab_source
        ms.m_data.update()
    
            
    def retrieve_coordinate(self, sec):
        """Retrieve the coordinates of the section avoiding duplicates"""
        
        sec.push()
        x, y, z, d = [],[],[],[]

        tot_points = 0
        connect_next = False
        for i in range(int(h.n3d())):
            present = False
            x_i = h.x3d(i)
            y_i = h.y3d(i)
            z_i = h.z3d(i)
            d_i = h.diam3d(i)
            # Avoiding duplicates in the sec
            if x_i in x:
                ind = len(x) - 1 - x[::-1].index(x_i) # Getting the index of last value
                if y_i == y[ind]:
                    if z_i == z[ind]:
                        present = True
                    
            if not present:
                k =(x_i, y_i, z_i)
                x.append(x_i)
                y.append(y_i)
                z.append(z_i)
                d.append(d_i)                
        h.pop_section()
        #adding num 3d points per section
        self.n3dpoints_per_sec[sec.name()] = len(d)
        return (np.array(x),np.array(y),np.array(z),np.array(d))



# * Copyright (C) Tue Jan  5 10:10:19 GMT 2010 - Michele Mattioni:
# *  
# * This file is part of NeuronVisio
# * 
# * NeuronVisio is free software: you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation, either version 3 of the License, or
# * (at your option) any later version.
#
# * NeuronVisio is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
#
# * You should have received a copy of the GNU General Public License
# * along with NeuronVisio.  If not, see <http://www.gnu.org/licenses/>.

#@PydevCodeAnalysisIgnoren
import os
"""
This is a check to make sure the sip and the QStrings play nicely in Windows, 
where the PySide is using the new Python API (Python 3)
http://www.mail-archive.com/matplotlib-users@lists.sourceforge.net/msg19702.html
http://stackoverflow.com/questions/1400858/how-to-create-qstring-in-pyqt4
This could be easily removed when we move to Python 3
"""
if os.name == 'nt':
    import sip
    sip.setapi('QString', 2)
    sip.setapi('QVariant', 2)
    from PyQt4 import QtGui, QtCore, uic
    from PyQt4.QtCore import *

from subprocess import call
from manager import SynVecRef
os.environ['ETS_TOOLKIT'] = 'qt4'
import logging
FORMAT = '%(levelname)s %(name)s %(lineno)s   %(message)s'
logging.basicConfig(level=logging.INFO, format=FORMAT)
logger = logging.getLogger(__name__)

import sys
sys.path.append(os.path.dirname(__file__)) 

if os.name != 'nt':
    from PyQt4 import QtGui, QtCore, uic
    from PyQt4.QtCore import Qt

import numpy as np

import matplotlib as mpl
if mpl.backends.backend is None: 
    mpl.use('Qt4Agg')
elif mpl.backends.backend != 'Qt4Agg':
    message = """You must use the Qt4 backend to be able to use  Neuronvisio. 
    Check your backend in ~/.matplotlib/matplotlibrc and set it to Qt4Agg"""
    logger.warning(message)
mpl.interactive(True)

from neuron import h
h.load_file("stdrun.hoc")

# Visio

from visio import Visio
import manager
import res # icons

# ModelDb
from modeldb.ModelDB import Models

class Controls():
    """Main class Neuronvisio"""
    def __init__(self):
        app = QtGui.QApplication.instance()
        self.ui_dir = 'ui'
        # Loading the UI
        self.ui = uic.loadUi(os.path.join(os.path.dirname(__file__), 
                                          self.ui_dir,
                                          "neuronvisio.ui"))
        
        # Connecting
        self.ui.Plot3D.connect(self.ui.Plot3D, 
                                     QtCore.SIGNAL('clicked()'), self.launch_visio)
        self.ui.plot_vector_btn.connect(self.ui.plot_vector_btn,
                                         QtCore.SIGNAL('clicked()'), self.plot_vector)
        self.ui.init_btn.connect(self.ui.init_btn,
                                 QtCore.SIGNAL('clicked()'), self.init)
        self.ui.run_btn.connect(self.ui.run_btn,
                                QtCore.SIGNAL('clicked()'), self.run)
        self.ui.dtSpinBox.connect(self.ui.dtSpinBox, 
                                  QtCore.SIGNAL('valueChanged(double)'), 
                                  self.dt_changed)
        self.ui.tstopSpinBox.connect(self.ui.tstopSpinBox, 
                                     QtCore.SIGNAL('valueChanged(double)'), 
                                     self.tstop_changed)
        self.ui.vSpinBox.connect(self.ui.vSpinBox, 
                                 QtCore.SIGNAL('valueChanged(double)'), 
                                     self.v_changed)
        self.ui.create_vector.connect(self.ui.create_vector,
                                      QtCore.SIGNAL('clicked()'), 
                                      self.create_vector)
        self.ui.actionAbout.connect(self.ui.actionAbout,
                                      QtCore.SIGNAL('triggered()'), 
                                      self.about)
        self.ui.timelineSlider.connect(self.ui.timelineSlider,
                                         QtCore.SIGNAL("valueChanged(int)"),
                                         self.on_timeline_value_changed)
        self.ui.animationTime.connect(self.ui.animationTime,
                                      QtCore.SIGNAL('returnPressed()'),
                                      self.on_animation_time_return_pressed)
        self.ui.actionLoad.connect(self.ui.actionLoad, 
                                   QtCore.SIGNAL("triggered()"),
                                   self.load_hdf)
        self.ui.actionSave.connect(self.ui.actionSave,
                                   QtCore.SIGNAL("triggered()"),
                                   self.save_hdf)
        self.ui.tabWidget.connect(self.ui.tabWidget, 
                                  QtCore.SIGNAL('currentChanged(int)'),
                                  self.populate_treeview_model)
        self.ui.tree_models.connect(self.ui.tree_models, 
                                    QtCore.SIGNAL('itemSelectionChanged ()'),
                                    self.select_model_treeview)
        self.ui.load_model_btn.connect(self.ui.load_model_btn, 
                                    QtCore.SIGNAL('clicked()'),
                                    self.load_selected_model)
        
        
        ### Connection with the console
        widgetDic = {'dt' : self.ui.dtSpinBox, 
                     'tstop' : self.ui.tstopSpinBox,
                     'v_init' : self.ui.vSpinBox, 
                     'time_label' : self.ui.time_label
                    }
        self.timeLoop = Timeloop(widgetDic)
        self.timeLoop.start()
        
        
        ### Manager class 
        self.manager = manager.Manager()
        self.path_to_hdf = None
        self.visio = None
        self.tab_model_already_populated = False  
        self.ui.show()
        # Start the main event loop.
        #app.exec_()
        
        self.AUTHORS = 1
        self.YEAR = 0
        self.TITLE = 2
        self.ID = 3
        # Dictionary to old the models class for the ModelDb integration
        self.models = None
        
    def populate_treeview_model(self, index):
        """populate the tree view and the scroll_area when the tab is 
        activated"""
        if not self.tab_model_already_populated and index == 3 :
            
            self.models =  Models()
            # Populating the treeview with the dictionary
            for model_name in self.models.get_model_names():
                model = self.models.get_model(model_name)
                model_item = QtGui.QTreeWidgetItem(self.ui.tree_models, 'Models')
                model_item.setText(self.YEAR, model.get_year())
                model_item.setText(self.AUTHORS, model.get_authors())
                model_item.setText(self.TITLE, model.get_title())
                model_item.setData(self.ID, 0, model.get_id())
                
                # tooltip
                cols = self.ui.tree_models.columnCount()
                for i in range (cols):
                    tooltip = model.get_tooltip()
                    model_item.setToolTip(i, tooltip)
            
                
            #Resizing the column.
            #self.ui.tree_models.resizeColumnToContents(0)
            self.ui.tree_models.resizeColumnToContents(self.YEAR)
            self.ui.tree_models.resizeColumnToContents(self.TITLE)
            self.ui.tree_models.resizeColumnToContents(self.ID)
            self.ui.textBrowser_readme.clear()
            self.ui.textBrowser_readme.insertPlainText("No model selected.")
            self.tab_model_already_populated = True #we populated only once.
        
    def select_model_treeview(self):
        """Synch the README and the modelOverview with the selected model."""
        mod = self._retrieve_selected_model()
        if mod:
            readme = mod.get_readme_html()
            overview = mod.get_overview()
            self.ui.textBrowser_readme.clear()
            self.ui.textBrowser_readme.insertHtml(readme)            
            self.ui.textBrowser_model_overview.clear()
            self.ui.textBrowser_model_overview.insertHtml(overview)
       
    def _retrieve_selected_model(self):
        "Return the model selected in the "
        items = self.ui.tree_models.selectedItems()
    
        if items:
            selected_item = items[0] #first element
            model_id = int(selected_item.text(self.ID))
            models_name = self.models.get_model_names()
            for name in models_name:
                mod = self.models.get_model(name)
                if model_id == mod.get_id():
                    return mod
        else:
            logging.info('No model selected!')
            return None
       
    def load_selected_model(self):
        "Load the model selected in the treeview."
        
        mod = self._retrieve_selected_model()
        if mod:
            model_path = mod.download_model()
            # tooltip
            cols = self.ui.tree_models.columnCount()
            items = self.ui.tree_models.selectedItems()
            model_item = items[0]
            for i in range (cols):
                tooltip = mod.get_tooltip()
                model_item.setToolTip(i, tooltip)
            self.run_extracted_model(mod)

    # create the command line to compile mod files into nrnmech.dll and launch it. command line is
    # <cygwin-dir>\bin\bash.exe -c "cd <model-dir>; /usr/bin/sh -c '<nrnhome>/lib/mknrndll.sh <nrnhome>'"
    def windows_compile_mod_files(self, model_dir):
        import _winreg
        k1=_winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, "SOFTWARE\\Cygwin\\setup")
        s1=_winreg.QueryValueEx(k1, 'rootdir')[0]
        _winreg.CloseKey(k1)

        k2=_winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, "SOFTWARE\\NEURON\\nrn72") 
        s2=_winreg.QueryValueEx(k2, 'Install_Dir')[0]
        s2=s2.replace('\\', '/')
        _winreg.CloseKey(k2)

        cmd=s1+"\\bin\\bash.exe"
        arg="cd "+model_dir+";/usr/bin/sh -c '" + s2 + "/lib/mknrndll.sh " + s2 + "'"
        import subprocess
        subprocess.Popen([cmd, '-c', arg], stdin=subprocess.PIPE).communicate(input="\r\n")

    def run_extracted_model(self, mod):
        model_dir = mod.get_dir()
        if os.path.exists(os.path.join (model_dir, 'mosinit.hoc')):
            old_dir = os.path.abspath(os.getcwd())
            logger.info("Path changed to %s" %(os.path.abspath(model_dir)))
            os.chdir(model_dir)
            try:
                # If windows
                if os.name == 'nt':                
                    self.windows_compile_mod_files('.')
                    from neuron import h
                    h.nrn_load_dll('./nrnmech.dll')
                else: # Anything else.
                    call(['nrnivmodl'])
                    import neuron            
                    neuron.load_mechanisms('./')
                from neuron import gui # to not freeze neuron gui
                from neuron import h
                logger.info("Loading model in %s" %model_dir)
                h.load_file('mosinit.hoc')
            except Exception as e:
                logger.warning("Error running model: "+e.message)
            logger.info("Path changed back to %s" %old_dir)
            os.chdir(old_dir)
        else: 
            response = """We didn't find any mosinit.hoc . Unfortunately we can't 
            automatically run the model. Check the README, maybe there is an 
            hint."""
            logging.warning(response)
            path_info = "You can find the extracted model in %s" %model_dir
            mod.browse()
            logging.info(path_info)
    
    def load_hdf(self, path_to_hdf=None):
    
        if path_to_hdf != None:
            self.path_to_hdf=os.path.abspath(path_to_hdf)
        
        else:
            filename = QtGui.QFileDialog.getOpenFileName()
            if filename:
                self.path_to_hdf = str(filename)
        
        if self.path_to_hdf != None:
            
            self.manager.load_from_hdf(self.path_to_hdf)
            self.update_tree_view()
            msg = "Loaded db: %s" % self.path_to_hdf
            self.ui.statusbar.showMessage(msg, 3500)
            # Enablig the Animation button
            self.animation()
            # Disabling all the rest
            self.ui.init_btn.setEnabled(False)
            self.ui.run_btn.setEnabled(False)
            self.ui.create_vector.setEnabled(False)
            
        
    def save_hdf(self):
        if not self.path_to_hdf:
            filename = QtGui.QFileDialog.getSaveFileName()
            self.path_to_hdf = str(filename) # It will go with python 3
            if self.path_to_hdf != None:
                self.manager.save_to_hdf(self.path_to_hdf)
                msg = "Saved hdf file: %s" % self.path_to_hdf
                self.ui.statusbar.showMessage(msg, 3500)
        
    def launch_visio(self):
        msg = "Plotting..."
        self.ui.statusbar.showMessage(msg, 3500)
        if self.visio == None:
            
            # Checking there are sections in the model.
            i = 0
            for sec in h.allsec():
                i += 1
            
            if i > 0:
                self.visio = Visio(self.ui.sec_info_label, self.manager)
                self.visio.draw_model()
                self.ui.selected_section.setEnabled(True)
            else:
                msg = """No model found, no section created. You need 
                to have at least one."""
                logger.warning(msg)
        else:
            #Raise the visio window
            self.visio.container.show()
        # Enabling the animation
        try:
            self.animation()
        except KeyError:
            # No simulation run an nothing loaded.
            # just pass
            pass
        
    
    def init(self):
        """Set the vm_init from the spin button and prepare the simulator"""
        
        if not self.manager.refs.has_key('VecRef') :
            logger.info("No vector Created. Create at least one vector to run the simulation")
            return False
        else:
            v_init = self.ui.vSpinBox.value()
            # Set the v_init
            h.v_init = v_init
            h.finitialize(v_init)
            h.fcurrent()
        
            # Reset the time in the GUI
            self.ui.time_label.setNum(h.t)
            return True
            
    
    def run(self):
        """Run the simulator till tstop"""
        
        #Initializing
        if self.init():
            # Run
            msg = "Running simulation. It will take a while maybe..."
            self.ui.statusbar.showMessage(msg, 5000)
            while h.t < h.tstop:
                h.fadvance()
                
                self.ui.time_label.setText("<b>" + str(h.t) + "</b>")

                    
    def tstop_changed(self):
        
        h.tstop = self.ui.tstopSpinBox.value()
        
        
    def dt_changed(self):
        
        h.dt = self.ui.dtSpinBox.value()
    
    def v_changed(self):
        
        h.v_init = self.ui.vSpinBox.value()
        
    def plot_vector(self):
        
        items = self.ui.treeWidget.selectedItems()
        
        x = None
        
        # Plot legend if required
        legend_status = self.ui.legend.isChecked() #return True if toggled.
        
        points_status = self.ui.points.isChecked()
        # Retrieve the fig num
        fig_num = self.ui.fig_num_spinBox.value()
        
        for item in items:
            if item.childCount() == 0: # Leaf, so it is the variable to plot
                
                sectionItem = item.parent()
                sectionName = str(sectionItem.text(0)) #Column used
                var = str(item.text(0))
                detail = str(item.text(1))
                
                groupName = str(sectionItem.parent().text(0))
                x = self.manager.groups[groupName]
                key = sectionName + "_" + var
                vecs_to_plot = { key : item.vec}
                self.manager.plot_vecs(vecs_to_plot, x=x, legend=legend_status, 
                              figure_num=fig_num, points=points_status)
    
    def create_vector(self):
        
        var = self.ui.var.text()
        if var.isEmpty():

            msgBox = QtGui.QMessageBox()
            msgBox.setText("No var specified.")
            msgBox.setIcon(QtGui.QMessageBox.Warning)
            msgBox.exec_()
 
        else:
            if self.ui.all_sections.isChecked():
                allCreated = self.manager.add_all_vecRef(str(var))
            elif self.ui.selected_section.isChecked():
                if self.visio.selected_cyl is not None:
                    sec = self.visio.cyl2sec[self.visio.selected_cyl]
                    self.manager.add_vecRef(str(var), sec)
                else:
                    msgBox = QtGui.QMessageBox()
                    msgBox.setText("<b>No vector has been created.</b>")
                    msg = "You need to select the section where you want to create the vector"
                    msgBox.setInformativeText(msg)
                    msgBox.setIcon(QtGui.QMessageBox.Warning)
                    msgBox.exec_()
        self.update_tree_view()

    def get_unique_parent(self, name, parentItem = None):
        """Search the name in the treeview and return the qtElement.
        Raise an exception if not unique"""
        search = self.ui.treeWidget.findItems(name , 
                                                Qt.MatchFixedString)
        root_item = None
        if len(search) == 0: # We create the group
            root_item = None
            if parentItem is None:
                root_item = QtGui.QTreeWidgetItem(self.ui.treeWidget)
            else:
                root_item = QtGui.QTreeWidgetItem(parentItem)
            root_item.setText(0, name)
            
        elif len(search) == 1:
            root_item = search[0]
            
        else:
            error = "ERROR - too many match: %d. Group Name not \
            unique." %len(search)
            raise NameError(error)
        
        return root_item
                
    def insert_item_treeview(self, groupName, section_name, vecs, 
                             details = None):
        """Insert a new item in the treewidget. 
        Items are grouped by types. If a new type is provided a new group is added.
        Items are then grouped by section.
        In one section more than one variable is allowed.
        Each variable can have a detail associated in a dictionary form """
        group_root = self.get_unique_parent(groupName)
        sec_root = self.get_unique_parent(section_name, 
                                          parentItem = group_root)
        for var,vec in vecs.iteritems():
            item = ItemRef(sec_root, vec)
            item.setText(0, var)
            item.setText(1, details)    
            sec_root.addChild(item)
            
    
    def insert_refs_in_treeview(self):
        for group, ref_list in self.manager.refs.iteritems():
            for ref in ref_list:
                self.insert_item_treeview(group, 
                                          ref.sec_name, 
                                          ref.vecs, 
                                          ref.detail)
            
                        
    def update_tree_view(self):
        # Fill the treeview wit all the vectors created
        #Clear all the row
        self.ui.treeWidget.clear()
        
        self.insert_refs_in_treeview()

    
    def animation(self):
        
        self.ui.timelineSlider.setRange(0, 
                                          len (self.manager.groups['t']))
        self.ui.timelineSlider.setEnabled(True)
        self.ui.show()
    
    
    
    def select_sections(self, list_of_sections):
        """Select an arbitrary number of sections from the 
        command line.
        
        
        :param: list_of_section - the list of sections to select. Each section should
                          be given by the name (sec.name() from Neuron point 
                          of view. 
        """
        if self.visio is not None:
            selection_scalar = self.visio.get_selection_scalar(list_of_sections)
            self.visio.redraw_color(selection_scalar, 'v')
            self.visio.update_sections_info(list_of_sections)
        else:
            logger.warning("You have to launch the 3D Visio window first!")
    
    def on_animation_time_return_pressed(self):
        "Getting the value from the text"
        time = self.ui.animationTime.text()
        try:
            time = int (time)
            time_list = self.manager.groups['t']
            time_point_indx = 0
            # If it's a vector on the just ran sim.
            if hasattr(time_list, 'to_python'):
                time_list = time_list.to_python()
                time_list = np.around(time_list, 3)
                
                time_point_indx = np.where(time_list==time)[0]
            # If it's a numpy array saved on the disk
            else:
                rounded = time_list.read().round(3)
                time_point_indx = np.where(rounded==time)[0]
                        
            self.sync_visio_3d(time_point_indx)
            self.ui.timelineSlider.setValue(time_point_indx)
        except:
            logger.warning("Value not present in the array.")
    
    def on_timeline_value_changed(self):
        """Draw the animation according to the value of the timeline"""
        
        time_point_indx = self.ui.timelineSlider.value()
        self.sync_visio_3d(time_point_indx)    
    
    def sync_visio_3d(self, time_point_indx):
        
        var = self.ui.varToShow.text()
        var = str(var) # This will go with Py3
#        
#        #Update the label on the scale
        
        if len (self.manager.groups['t']) == time_point_indx:
            time_point_indx = time_point_indx - 1 # Avoid to go out of scale
        time = self.manager.groups['t'][time_point_indx]
        self.ui.animationTime.setText(str(time))
        
        
        start_value = float(self.ui.startValue.text())
        end_value = float(self.ui.endValue.text())

        self.visio.show_variable_timecourse(var, time_point_indx, 
                                            start_value, end_value)
    
    def about(self):
        
        self.aboutUi = uic.loadUi(os.path.join(os.path.dirname(__file__),
                                               self.ui_dir,
                                               "qtAbout.ui"))
        import neuronvisio
        name = '<font size=24><b>Neuronvisio %s<b><font>' %neuronvisio.__version__
        authors = '%s' %neuronvisio.__authors__
        
        self.aboutUi.name.setText(name)
        self.aboutUi.authors.setText(authors)    
        self.aboutUi.show()
            
class ItemRef(QtGui.QTreeWidgetItem):
    def __init__(self, sec_root, vec):
        QtGui.QTreeWidgetItem.__init__(self, sec_root) # >1000 if custom.
        self.vec = vec
    
    
        
class Timeloop(QtCore.QThread):
    """Daemon thread to connect the console with the gui"""
    def __init__(self, widgetDic, parent = None):
        QtCore.QThread.__init__(self, parent)
        self.widgetDic = widgetDic
        
        
    def run(self):
        """Update the gui interface"""
        while True:
            self.sleep(1) #check every sec
            
            if h.dt != self.widgetDic['dt'].value():
                self.widgetDic['dt'].setValue(h.dt)
            if h.tstop != self.widgetDic['tstop'].value():
                self.widgetDic['tstop'].setValue(h.tstop)
            if h.v_init != self.widgetDic['v_init'].value():
                self.widgetDic['v_init'].setValue(h.v_init)
            
                
            
            
